<?php

namespace loyalsoft;

header('Cache-Control: no-store');                                              # 防御缓存设施
header('Host:' . PHP_OS . "-" . (isset($_SERVER['SERVER_ADDR']) ? $_SERVER['SERVER_ADDR'] : ""));   # 这里提取服务器地址返给客户端，方便留意负载均衡/弹性扩容的效果


$GLOBALS['OS'] = "win32";                                                       # os,      used by JsonUtil::encode()

defined("ROOTDIR") or define("ROOTDIR", __DIR__);                               # 定义项目根目录

/**
 * 获取项目根路径URL(限制条件,本方法所在的文件也必须在项目根路径下wg)
 * @author gwang
 * @return string
 */
function getRootURL() {
    $self = substr(__FILE__, strlen($_SERVER['DOCUMENT_ROOT']));
    $self = str_replace('\\', '/', $self);
    $uri = substr($self, 0, strrpos($self, '/') + 1);
    $url = "http://" . $_SERVER['HTTP_HOST'] . $uri;
    return $url;
}

/**
 * 一种优化方案, 如果部署环境可以支持动态代码生成,将会减少与redis网络通信的次数. 
 * (在云服务器实验中,开了不如不开效率高￣□￣｜｜,而且文件模式需要在多台机子中分发.-gwang2017)
 * 经验证,复杂逻辑(会多次读取配置信息时)可以有效改善性能. -gwang 2020.11.24 
 * @var boolean (部署环境)是否允许代码生成
 */
define('CodeGen_Enabled', false)
        and CodeGen_Enabled
        and define('CodeGen_Folder', ROOTDIR . '/../../CodeGen/');              # 代码生成功能的输出位置

require_once ROOTDIR . '/AutoLoad.php';                                         # 初始化框架自动加载机制 

/**
 * get dao instance,
 * @version 2019年12月23日 经过梦幻星工场2一年多的使用, 未发现特别明显的bug. 决定侧重使用. -- 王刚
 *           2017.06.23     第一版 学习自禅道的开源框架.    -- 王刚
 * @staticvar type $a
 * @return \dao  注意: dao并非什么好东西,只是一些改进尝试, 稳定性, 性能, 效率尚未得到证明.-gwang 2017.06.23
 */
function daoInst() {
    static $a = null;
    if (is_null($a)) {
        $a = new dao();                                                         # 结束请求的时候自动回收,无需主动写关闭代码
        $a->connectDB(config::Inst());                                          # 建立链接,传入配置文件
    }
    return $a;
}

/**
 * @return CRedisUtil 获取全局MEM单例
 */
function gMem() {
    static $cmem = null;
    if ($cmem == null) {
        $nosql = config::Inst()->nosql;
//        $use_ext = true;                # 使用扩展,测试用(暂时仅支持php7)                       
//        if ($use_ext) {
//            $cmem = new \Redis();       # 还是需要重写下, 之前的CRedisUtil已经增加了json_encode/decode.
//            $cmem->connect($nosql->host, $nosql->port);
//            $cmem->auth($nosql->pwd);
//        } else {
        $cmem = new CRedisUtil();
        $cmem->conn($nosql->host, $nosql->port, $nosql->pwd);
//        }
    }
    return $cmem;
}

/**
 * 简化提取全局变量的写法
 * @return globalsettings
 */
function glc() {
    return GameConfig::globalsettings();
}

/**
 * @return loyalsoft\Req 全局访问
 */
function req() {
    return Req::Ins();
}

default_timezone();                                                             # 全局范围内启用默认时区 中国时区

$zoneid = "1";                                                                  # 定义全局变量zoneid

/**
 * 从QueryString初始化分区Id
 * @global type $zoneid
 */
function InitZoneId() {
    global $zoneid;
    if (isset($_REQUEST['zoneid'])) {
        $zoneid = $_REQUEST['zoneid'];                                          # 从参数提取到全局变量.
    }
}

/**
 * 直接返回错误resp并结束运行
 * @param type $err
 * @param type $msg
 */
function Err($err, $msg = "") {
    if (ErrCode::ok === $err) {
        CLog::err("Err()方法的参数err==ok, 那你还调个蛋的Err()!!!");
    }
    if (Index::$isDebugging) {                                                  # 组织调试信息       
        Index::$debugMsg .= "<br/><font color='red'>发生错误逻辑中断($err)：$msg</font>"; # 错误信息用红色显示    
        Index::$debugMsg .= '<br/>内存分配:' . number_format(memory_get_peak_usage() / 1024) . 'kb,';
        Index::$debugMsg .= '内存占用:' . number_format(memory_get_usage() / 1024) . 'kb';  # 内存使用数量(kb) 
    }
    if (is_string($err)) {                                                      # 防御Err中直接输入字符串的货
        $msg = "发生错误($err)直接中断逻辑执行." . $msg;
        CLog::err($msg);
        $err = ErrCode::err_innerfault;
    }
    exit(Index::OverPass(Resp::err($err)));                                     # 直接返回错误信息  
}

/**
 * 断言快速访问
 * @param bool $condition
 * @param int/str $err
 */
function my_Assert($condition, $err) {
    DebugHelper::assert($condition, $err);
}

/**
 * 功能已过期 
 */
function absolute() {
    return Err(ErrCode::err_method_obsoleted);
}

/**
 * 带换行符输出
 */
function echoLine() {
    $cli = (php_sapi_name() === 'cli');
    $args = func_get_args();
    echo implode(" ", $args) . ( $cli ? PHP_EOL : "<br/>");                     # CLI下追加\n, html下追加<br>
}

if (GAME_ONLINE) {                                                              # 设置脚本执行的超时时间
    set_time_limit(25);                                                         # 外网不超过25秒
} else {
    set_time_limit(10);                                                          # 内网不超过5秒
}

